#include "common/device_info.h"
#include "datafox/communication/module_hardware.h"
#include "common/software_version.h"

DeviceInfo::DeviceInfo()
{
    clear();
}

DeviceInfo::~DeviceInfo()
{
}

void DeviceInfo::clear()
{
    m_versionBootloader = "";
    m_bootloaderMode = false;
    m_versionFirmware = "";

    m_serialNumber = 0;

    m_hardwareInfo.reset();

    m_versionFingerprint = "";
    m_versionUSBHost = "";
    m_versionGPS = "";

    m_macAddress = "";

    m_softwareInfo.clear();
    m_passwordKey = "";

    m_firmwareOptions.clear();

    m_accessControlMasterId = "";
    m_timeboyGroupId = "";

    m_mainInterfaceType = 0;
}

void DeviceInfo::setVersionBootloader( const std::string &version )
{
    m_versionBootloader = version;
}

std::string DeviceInfo::versionBootloader() const
{
    return m_versionBootloader;
}

void DeviceInfo::setBootloaderMode( bool active )
{
    m_bootloaderMode = active;
}

bool DeviceInfo::bootloaderMode() const
{
    return m_bootloaderMode;
}

void DeviceInfo::setHardwareInfo( const HardwareInfo &hardwareInfo )
{
    m_hardwareInfo = hardwareInfo;
}

HardwareInfo &DeviceInfo::hardwareInfo()
{
    return m_hardwareInfo;
}

void DeviceInfo::setFileHardwareInfo( const FileHardwareInfo &file )
{
    m_hardwareInfo.setFileHardwareInfo( file );
}

FileHardwareInfo &DeviceInfo::fileHardwareInfo()
{
    return m_hardwareInfo.fileHardwareInfo();
}

listItemBoard DeviceInfo::collectModuleIds( const setModuleId &ids )
{
    return m_hardwareInfo.collectModuleIds( ids );
}

void DeviceInfo::setVersionFirmware( const std::string &version )
{
    m_versionFirmware = version;
}

std::string DeviceInfo::versionFirmware() const
{
    return m_versionFirmware;
}

void DeviceInfo::setSoftwareInfo( const SoftwareInfo &softwareInfo )
{
    m_softwareInfo = softwareInfo;
}

SoftwareInfo &DeviceInfo::softwareInfo()
{
    return m_softwareInfo;
}

void DeviceInfo::setFirmwareOptions( const FirmwareOptions &firmwareOptions )
{
    m_firmwareOptions = firmwareOptions;
}

FirmwareOptions &DeviceInfo::firmwareOptions()
{
    return m_firmwareOptions;
}

void DeviceInfo::setSerialNumber( uint32_t serial )
{
    m_serialNumber = serial;
}

uint32_t DeviceInfo::serialNumber() const
{
    return m_serialNumber;
}

void DeviceInfo::setPasswordKey( const std::string &key )
{
    m_passwordKey = key;
}

std::string DeviceInfo::passwordKey() const
{
    return m_passwordKey;
}

void DeviceInfo::setVersionFingerprint( const std::string &version )
{
    m_versionFingerprint = version;
}

std::string DeviceInfo::versionFingerprint() const
{
    return m_versionFingerprint;
}

void DeviceInfo::setVersionUSBHost( const std::string &version )
{
    m_versionUSBHost = version;
}

std::string DeviceInfo::versionUSBHost() const
{
    return m_versionUSBHost;
}

void DeviceInfo::setVersionGPS( const std::string &version )
{
    m_versionGPS = version;
}

std::string DeviceInfo::versionGPS() const
{
    return m_versionGPS;
}

void DeviceInfo::setVersionTCPIP( const std::string &version )
{
    m_versionTCPIP = version;
}

std::string DeviceInfo::versionTCPIP() const
{
    return m_versionTCPIP;
}

void DeviceInfo::setMacAddress( const std::string &address )
{
    m_macAddress = address;
}

std::string DeviceInfo::macAddress() const
{
    return m_macAddress;
}

void DeviceInfo::setAccessControlMasterId( const std::string &id )
{
    m_accessControlMasterId = id;
}

std::string DeviceInfo::accessControlMasterId() const
{
    return m_accessControlMasterId;
}

void DeviceInfo::setTimeboyGroupId( const std::string &id )
{
    m_timeboyGroupId = id;
}

std::string DeviceInfo::timeboyGroupId() const
{
    return m_timeboyGroupId;
}

void DeviceInfo::setBoardApplication( const std::string &val )
{
    m_boardApplication = val;
}

std::string DeviceInfo::boardApplication() const
{
    return m_boardApplication;
}

void DeviceInfo::setBoardPlace_0_Output_1_Mode( const std::string &mode )
{
    m_boardPlace_0_Output_1_Mode = mode;
}

std::string DeviceInfo::boardPlace_0_Output_1_Mode() const
{
    return m_boardPlace_0_Output_1_Mode;
}

void DeviceInfo::setBoardPlace_0_Serial_1_Use( const std::string &use )
{
    m_boardPlace_0_Serial_1_Use = use;
}

std::string DeviceInfo::boardPlace_0_Serial_1_Use() const
{
    return m_boardPlace_0_Serial_1_Use;
}

void DeviceInfo::setBoardPlace_0_Serial_2_Use( const std::string &use )
{
    m_boardPlace_0_Serial_2_Use = use;
}

std::string DeviceInfo::boardPlace_0_Serial_2_Use() const
{
    return m_boardPlace_0_Serial_2_Use;
}

void DeviceInfo::setTimeStartOS( const std::string &val )
{
    m_timeStartOS = val;
}

std::string DeviceInfo::timeStartOS() const
{
    return m_timeStartOS;
}

void DeviceInfo::setOffTimeLong( const std::string &val )
{
    m_offTimeLong = val;
}

std::string DeviceInfo::offTimeLong() const
{
    return m_offTimeLong;
}

void DeviceInfo::setOffTimeShort( const std::string &val )
{
    m_offTimeShort = val;
}

std::string DeviceInfo::offTimeShort() const
{
    return m_offTimeShort;
}

void DeviceInfo::setOnTime( const std::string &val )
{
    m_onTime = val;
}

std::string DeviceInfo::onTime() const
{
    return m_onTime;
}

void DeviceInfo::set_restore_ac_power_loss( const std::string &val )
{
    m_restoreAcPowerLoss = val;
}

std::string DeviceInfo::get_restore_ac_power_loss() const
{
    return m_restoreAcPowerLoss;
}

void DeviceInfo::setAccessModulesState( const std::string &val )
{
    m_accessModulesState = val;
}

std::string DeviceInfo::getAccessModulesState() const
{
    return m_accessModulesState;
}

bool DeviceInfo::importStructHardwareInfo( const StructHardwareInfo &info )
{  
    return m_hardwareInfo.importStructHardwareInfo( info );
}

StructHardwareInfo DeviceInfo::structHardwareInfo()
{
    return m_hardwareInfo.exportStructHardwareInfo();
}

void DeviceInfo::setStructSoftwareInfo( const StructSoftwareInfo &info )
{
    m_softwareInfo.setStructSoftwareInfo( info );
}

StructSoftwareInfo DeviceInfo::structSoftwareInfo() const
{
    return m_softwareInfo.structSoftwareInfo();
}

StructSoftwareInfo DeviceInfo::checkInfo( const StructSoftwareInfo &info )
{
    return m_softwareInfo.check( info );
}

int DeviceInfo::SoftwareInfoToString( const StructSoftwareInfo &swInfo, std::string &info )
{
    return m_softwareInfo.toString( swInfo, info );
}

int DeviceInfo::compareInfos( const StructHardwareInfo &hwInfo, const StructSoftwareInfo &swInfo, const std::string &versionFirmware, bool checkBoardRevision )
{
    uint32_t bit;

    // Keine Hardwareinfo per Software gesetzt worden?
    if ( hwInfo.version == 0 )
    { 
        // Dieses Betrifft lediglich die Geraete: PZE-/ZK-MasterIV

        // Je nach Geraetetyp Standardkonfiguration annehmen.
        // Die meisten Optionen werden durch die (04.00.23) Firmware ermittelt und gesetzt.
        switch ( hwInfo.typeDevice )
        {
        case 0: //IV_VALUE_TYPE_DEVICE_PZE
            // Zusaetzlich Tastatur und Display sicherstellen
            const_cast<StructHardwareInfo &>( hwInfo ).typeDisplay = 1;//IV_VALUE_TYPE_DISPLAY_SMALL;
            const_cast<StructHardwareInfo &>( hwInfo ).typeKeypad  = 1;//IV_VALUE_TYPE_KEYPAD_9;
            break;
        case 1: //IV_VALUE_TYPE_DEVICE_ZK
            // Zusaetzlich Zutrittskontrolle sicherstellen
            const_cast<StructHardwareInfo &>( hwInfo ).componentRS485_1 = 1;
            const_cast<StructHardwareInfo &>( hwInfo ).typeExternalReader = 1;//IV_VALUE_TYPE_EXTERNAL_READER_GIS;
            break;
        }

        // Ist die ZK-Option in Verwendung?
        if ( hwInfo.administrationFlags & HIAF_HAS_ZK )
        {  
            // ZK-Option mit GIS Lesern.
            const_cast<StructHardwareInfo &>( hwInfo ).componentRS485_1 = 1;
            const_cast<StructHardwareInfo &>( hwInfo ).typeExternalReader = 1;
        }
    }

    // Die ersten 32 Geräte sind in deviceExtended abgelegt, die weiteren 32 in deviceExtended_2
    if ( hwInfo.typeDevice < 32 ) // IV_VALUE_TYPE_DEVICE_IPC_EVO_18_5
    {
        // Geraetetyp pruefen
        if ( ((1 << hwInfo.typeDevice) & swInfo.deviceExtended) == 0 )
        {
            m_errorText = "Der Geraetetyp wird nicht unterstuetzt.";
            return 0;
        }
    }
    else
    {
        // Geraetetyp pruefen
        if ( ((1 << (hwInfo.typeDevice - 32)) & swInfo.deviceExtended_2) == 0 )
        {
            m_errorText = "Der Geraetetyp wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Hardwareversion <= v3 müssen unabhängig von der Verfügbarkeit der Datenbank nach dem alten Schema behandelt werden.
    if ( checkBoardRevision || m_hardwareInfo.isMasterIV( hwInfo ) )
    {
        uint8_t type = 0;
        // Boardtyp pruefen
        // Bei der Switch müssen sich die case Fälle nach dem Hinzufügen der Version richten, da type dann die Bitposition der Softwareinfo bestimmt.
        // Der Boardtyp muss auch in der Datei commmon/software_info.h als Define angelegt werden!
        switch ( hwInfo.typeBoard )
        {
            case IV_VALUE_TYPE_BOARD_4_9: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_1_9: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_4_8: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_2_7: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_1_8: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_4_7: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_4_6: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_2_6: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_1_7: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_2_5: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_2_2: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_1_6: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_4_5: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_1_5: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_4_4: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_3_1: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_2_4: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_2_3: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_4_3: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_3_4: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_4_2: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_4_1: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_4_0: type++; BOOST_FALLTHROUGH;
            case IV_VALUE_TYPE_BOARD_3_0: type++; BOOST_FALLTHROUGH; // 10000000 0x80
            case IV_VALUE_TYPE_BOARD_2_1: type++; BOOST_FALLTHROUGH; //  1000000 0x40
            case IV_VALUE_TYPE_BOARD_2_0: type++; BOOST_FALLTHROUGH; //   100000 0x20
            case IV_VALUE_TYPE_BOARD_1_4: type++; BOOST_FALLTHROUGH; //    10000 0x10
            case IV_VALUE_TYPE_BOARD_1_3: type++; BOOST_FALLTHROUGH; //     1000 0x08
            case IV_VALUE_TYPE_BOARD_1_2: type++; BOOST_FALLTHROUGH; //      100 0x04
            case IV_VALUE_TYPE_BOARD_1_1: type++; BOOST_FALLTHROUGH; //       10 0x02
            case IV_VALUE_TYPE_BOARD_1_0:         //        1 0x01
                break;

            default:
                m_errorText = "Die Boardrevision wird nicht unterstuetzt.";
                return 0;
        }

        // Die ersten 32 Boardrevisionen sind in boardExtended abgelegt, die weiteren 32 in boardExtended_2
        if ( type < 32 )
        {
            if ( ((1 << type) & swInfo.boardExtended) == 0 )
            {
                m_errorText = "Die Boardrevision wird nicht unterstuetzt.";
                return 0;
            }
        }
        else
        {
            if ( ((1 << (type - 32)) & swInfo.boardExtended_2) == 0 )
            {
                m_errorText = "Die Boardrevision wird nicht unterstuetzt.";
                return 0;
            }
        }

        // B_1020: Verhinderung des Aufspielens von FW-Versionen < 04.03.15.06 auf Kombi-HW 4.8a, IOBox-HW 1.8a (wegen Flashbaustein)
        switch ( hwInfo.typeDevice )
        {
            case IV_VALUE_TYPE_DEVICE_PZE:
            case IV_VALUE_TYPE_DEVICE_AE:
            case IV_VALUE_TYPE_DEVICE_EVO_4_3:
            case IV_VALUE_TYPE_DEVICE_EVO_4_6_FLEX_KEY:
            {
                if ( hwInfo.typeBoard >= IV_VALUE_TYPE_BOARD_4_8 )
                {
                    SoftwarebuildVersion version( versionFirmware.c_str() );
                    // Die zu Übertragende Firmwareversion muss mindestens 04.03.15.06 sein.
                    if ( version < "04.03.15.06" )
                    {
                        m_errorText = "Die Firmwareversion ist fuer die Boardrevision gesperrt.";
                        return 0;
                    }
                }
                break;
            }

            case IV_VALUE_TYPE_DEVICE_EVO_2_8_PURE:
            case IV_VALUE_TYPE_DEVICE_IO_BOX:
            case IV_VALUE_TYPE_DEVICE_KYO_INLOC:
            case IV_VALUE_TYPE_DEVICE_MOBIL_BOX:
            case IV_VALUE_TYPE_DEVICE_EVO_3_5_PURE:
            case IV_VALUE_TYPE_DEVICE_EVO_5_0_PURE:
            {
                if ( hwInfo.typeBoard >= IV_VALUE_TYPE_BOARD_1_8 )
                {
                    SoftwarebuildVersion version( versionFirmware.c_str() );
                    // Die zu Übertragende Firmwareversion muss mindestens 04.03.15.06 sein.
                    if ( version < "04.03.15.06" )
                    {
                        m_errorText = "Die Firmwareversion ist fuer die Boardrevision gesperrt.";
                        return 0;
                    }
                }
                break;
            }
        }
    }

    // Workaround für Docking V1 / V2 und Fahrzeugdatenlogger V1 / V2
    // B_1042: Firmwareunterstützung älterer Platinenversionen (Docking/Fahrzeugdatenlogger) begrenzt
    switch ( hwInfo.typeDevice )
    {
        case IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK:
        case IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK:
        {
            SoftwarebuildVersion version( versionFirmware.c_str() );
            if ( hwInfo.typeBoard < IV_VALUE_TYPE_BOARD_2_6 )
            {
                // Die zu Übertragende Firmwareversion darf höchsten 04.03.10.x sein.
                if ( version > "04.03.10.99" )
                {
                    m_errorText = "Die Firmwareversion ist fuer die Boardrevision gesperrt.";
                    return 0;
                }
            }
            else
            {
                // Die zu Übertragende Firmwareversion muss mindestens 04.03.11.0 sein.
                if ( version < "04.03.11.0" )
                {
                    m_errorText = "Die Firmwareversion ist fuer die Boardrevision gesperrt.";
                    return 0;
                }
            }
            break;
        }
    }

    // Handelt es sich nicht um einen MasterIV.
    if ( !m_hardwareInfo.isMasterIV( hwInfo ) )
        return 1;

    // IO-Modul pruefen
    if ( hwInfo.optionIOModule && !swInfo.iomodule )
    {
        // IOModule bei folgenden Geraeten nicht moeglich!
        // Die gewaehlte Option ist unnoetig.
        if ( hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_PZE && 
            hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_TIMEBOY && 
            hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE )
        {
            m_errorText = "Das IO-Modul wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Timeboyanbindung pruefen
    if ( hwInfo.optionTBAdaption && !swInfo.docking )
    {
        // Anbindung Timeboy bei folgenden Geraeten nicht moeglich!
        // Die gewaehlte Option ist unnoetig.
        if ( hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_BDE &&
            hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_TIMEBOY &&
            hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE )
        {
            m_errorText = "Die Timeboyanbindung wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Comserver pruefen
    if ( hwInfo.optionComserver && !swInfo.comserver )
    {
        m_errorText = "Der Comserver-Modus wird nicht unterstuetzt.";
        return 0;
    }

    // Transponderwert schreiben pruefen
    if ( hwInfo.optionTPWrite && !swInfo.transponder_write )
    {
        // Wenn kein Transponder vorliegt, ist die Option unnoetig!
        // Auch wenn kein Display vorliegt!
        if ( hwInfo.typeTransponder != IV_VALUE_TYPE_TRANSPONDER_NONE && hwInfo.typeDisplay != IV_VALUE_TYPE_DISPLAY_NONE )
        {
            m_errorText = "Das Schreiben eines Transponders wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Http-Protokoll pruefen
    if ( hwInfo.optionHttp && !swInfo.http )
    {
        // Wenn kein Modem eingebaut, ist die Option unnoetig!
        if ( hwInfo.typeModemMC != IV_VALUE_TYPE_MODEM_MC_NONE )
        {
            m_errorText = "Das Http-Protokoll wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Sprachdatei pruefen
    if ( hwInfo.optionLanguage && !swInfo.language )
    {
        m_errorText = "Die Sprachdateien werden nicht unterstuetzt.";
        return 0;
    }

    // Update des Biokey pruefen
    if ( hwInfo.optionBiokeyUpdate && !swInfo.biokey_update )
    {
        // Wenn kein Biokey-Modul eingebaut, ist die Option unnoetig!
        if ( hwInfo.typeFingerprint != IV_VALUE_TYPE_FINGERPRINT_NONE )
        {
            m_errorText = "Das Update des Biokey wird nicht unterstuetzt.";
            return 0;
        }
    }

    // RS485 pruefen
    if ( hwInfo.componentRS485_0 && !swInfo.rs485 )
    {
        m_errorText = "Die RS485-Kommunikation wird nicht unterstuetzt.";
        return 0;
    }

    // TCPIP pruefen
    if ( hwInfo.typeTCPIP && (((1 << (hwInfo.typeTCPIP - 1)) & swInfo.tcpip)) == 0 )
    {
        m_errorText = "Die TCPIP-Kommunikation wird nicht unterstuetzt.";
        return 0;
    }

    // ModemMCxx pruefen
    if ( hwInfo.typeModemMC && (((1 << (hwInfo.typeModemMC - 1)) & swInfo.modem)) == 0 )
    {
        m_errorText = "Das verwendete Modem wird nicht unterstuetzt.";
        return 0;
    }

    // RS485 (intern) ermitteln
    if ( hwInfo.componentRS485_1 && !swInfo.access )
    {
        m_errorText = "Die Zutrittskontrolle wird nicht unterstuetzt.";
        return 0;
    }

    // Display pruefen
    if ( hwInfo.typeDisplay && ((1 << (hwInfo.typeDisplay - 1)) & swInfo.display) == 0 )
    {
        m_errorText = "Das verwendete Display wird nicht unterstuetzt.";
        return 0;
    }

    // Tastatur pruefen
    if ( hwInfo.typeKeypad && ((1 << (hwInfo.typeKeypad - 1)) & swInfo.keypad) == 0 )
    {
        m_errorText = "Die verwendete Tastatur wird nicht unterstuetzt.";
        return 0;
    }

    // Transponder pruefen
    if ( hwInfo.typeTransponder && ((1 << (hwInfo.typeTransponder - 1)) & swInfo.transponderExtended) == 0 )
    {
        m_errorText = "Der verwendete Transponderleser wird nicht unterstuetzt.";
        return 0;
    }

    // MMC pruefen
    if ( hwInfo.typeMMC && ((1 << (hwInfo.typeMMC - 1)) & swInfo.mmc) == 0 )
    {
        m_errorText = "Die verwendete MMC/USB-Host-Option wird nicht unterstuetzt.";
        return 0;
    }

    // Funk pruefen
    if ( hwInfo.typeRadio && ((1 << (hwInfo.typeRadio - 1)) & swInfo.radio) == 0 )
    {
        m_errorText = "Das verwendete Funkmodul wird nicht unterstuetzt.";
        return 0;
    }

    // GPS pruefen
    if ( hwInfo.typeGPS && ((1 << (hwInfo.typeGPS - 1)) & swInfo.gps) == 0 )
    {
        m_errorText = "Das verwendete GPS-Modul wird nicht unterstuetzt.";
        return 0;
    }

    // USB pruefen
    if ( hwInfo.componentUSB && !swInfo.usb )
    {
        m_errorText = "Die USB-Kommunikation wird nicht unterstuetzt.";
        return 0;
    }

    // DCF77 / Barcode (intern) pruefen
    if ( hwInfo.typeDCF77 && ((1 << (hwInfo.typeDCF77 - 1)) & swInfo.dcf77) == 0 )
    {
        m_errorText = "Das verwendete DCF77/Barcode (intern) - Modul wird nicht unterstuetzt.";
        return 0;
    }


    // DigitalIn pruefen
    if ( hwInfo.componentInAnalog )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( (bit & hwInfo.componentInAnalog) && !(bit & swInfo.analog_in) )
            {
                m_errorText = "Analoger Eingang wird nicht unterstuetzt.";
                return 0;
            }

            if ( bit == 0x8000000 )
                break;
        }
    }

    // DigitalIn pruefen
    if ( hwInfo.componentInDigital )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( (bit & hwInfo.componentInDigital) && !(bit & swInfo.digital_in) ) 
            {
                m_errorText = "Digitaler Eingang wird nicht unterstuetzt.";
                return 0;
            }

            if ( bit == 0x8000000 )
                break;
        }
    }

    // DigitalOut pruefen
    if ( hwInfo.componentOutDigital )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( (bit & hwInfo.componentOutDigital) && !(bit & swInfo.digital_out) )
            {
                m_errorText = "Digitaler Ausgang wird nicht unterstuetzt.";
                return 0;
            }

            if ( bit == 0x8000000 )
                break;
        }
    }

    // Fingerprint pruefen
    if ( hwInfo.typeFingerprint && ((1 << (hwInfo.typeFingerprint - 1)) & swInfo.fingerprint) == 0 )
    {
        m_errorText = "Das verwendete Fingerprint-Modul wird nicht unterstuetzt.";
        return 0;
    }

    // Externe Zutrittsleser pruefen
    if ( hwInfo.typeExternalReader && ((hwInfo.typeExternalReader & swInfo.access) == 0) )
    {
        m_errorText = "Die Externen Zutrittsleser werden nicht unterstuetzt.";
        return 0;
    }

    // Sollen die Setupoptionen geprueft werden
    if ( hwInfo.administrationFlags & HIAF_HAS_OPTIONS3 )
    {
        // SetupOption pruefen
        if ( hwInfo.optionsSetup )
        {
            for ( bit = 1; ; bit <<= 1 )
            {
                if ( (bit & hwInfo.optionsSetup) && !(bit & swInfo.setup_options) )
                {
                    m_errorText = "Softwareoption wird nicht unterstuetzt.";
                    return 0;
                }

                if ( bit == 0x8000 )
                    break;
            }
        }

    }

    // Sind die Optionen der Gruppe 4 zum Pruefen vorhanden?
    if ( hwInfo.administrationFlags & HIAF_HAS_OPTIONS4 )
    {

        // Touch pruefen
        if ( hwInfo.typeTouch && ((1 << (hwInfo.typeTouch - 1)) & swInfo.touch) == 0 )
        {
            m_errorText = "Das verwendete Touchpanel wird nicht unterstuetzt.";
            return 0;
        }

        // Sensor pruefen
        if ( hwInfo.typeSensor && ((1 << (hwInfo.typeSensor - 1)) & swInfo.sensor) == 0 )
        {
            m_errorText = "Der verwendete Sensor wird nicht unterstuetzt.";
            return 0;
        }

        // Batterie pruefen
        if ( hwInfo.componentBattery && !swInfo.battery )
        {
            m_errorText = "Die benoetigte Batterieueberwachung wird nicht unterstuetzt.";
            return 0;
        }

        // Ladeschaltung pruefen
        if ( hwInfo.componentCharge && !swInfo.charge )
        {
            m_errorText = "Die Akkuueberwachung (Ladeschaltung) wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Sind die Optionen der Gruppe 5 zum Pruefen vorhanden?
    if ( hwInfo.administrationFlags & HIAF_HAS_OPTIONS5 )
    {
        // Erweiterte Optionen pruefen
        if ( hwInfo.optionsPayable )
        {
            for ( bit = 1; ; bit <<= 1 )
            {
                if ( (bit & hwInfo.optionsPayable) && !(bit & swInfo.payable_options) )
                {
                    m_errorText = "Erweiterte Option wird nicht unterstuetzt.";
                    return 0;
                }

                if ( bit == 0x80 )
                    break;
            }
        }
    }

    return 1;
}

void DeviceInfo::setSystemFlags( uint32_t flags )
{
    m_firmwareOptions.setSystemFlags( flags );
}

uint32_t DeviceInfo::systemFlags() const
{
    return m_firmwareOptions.getSystemFlags();
}

void DeviceInfo::setTypeExternalReader( uint8_t type )
{
    m_firmwareOptions.setTypeExternalReader( type );
}

uint8_t DeviceInfo::getTypeExternalReader()
{
    return m_firmwareOptions.getTypeExternalReader();
}

void DeviceInfo::setOptionIOModule( uint8_t enabled )
{
    m_firmwareOptions.setOptionIOModule( enabled );
}

uint8_t DeviceInfo::getOptionIOModule()
{
    return m_firmwareOptions.getOptionIOModule();
}

void DeviceInfo::setOptionTBAdaption( uint8_t enabled )
{
    m_firmwareOptions.setOptionTBAdaption( enabled );
}

uint8_t DeviceInfo::getOptionTBAdaption()
{
    return m_firmwareOptions.getOptionTBAdaption();
}

void DeviceInfo::setOptionComServer( uint8_t enabled )
{
    m_firmwareOptions.setOptionComServer( enabled );
}

uint8_t DeviceInfo::getOptionComServer()
{
    return m_firmwareOptions.getOptionComServer();
}

void DeviceInfo::setOptionTPWrite( uint8_t enabled )
{
    m_firmwareOptions.setOptionTPWrite( enabled );
}

uint8_t DeviceInfo::getOptionTPWrite()
{
    return m_firmwareOptions.getOptionTPWrite();
}

void DeviceInfo::setOptionHTTP( uint8_t enabled )
{
    m_firmwareOptions.setOptionHTTP( enabled );
}

uint8_t DeviceInfo::getOptionHTTP()
{
    return m_firmwareOptions.getOptionHTTP();
}

void DeviceInfo::setOptionLanguage( uint8_t enabled )
{
    m_firmwareOptions.setOptionLanguage( enabled );
}

uint8_t DeviceInfo::getOptionLanguage()
{
    return m_firmwareOptions.getOptionLanguage();
}

void DeviceInfo::setOptionBiokeyUpdate( uint8_t enabled )
{
    m_firmwareOptions.setOptionBiokeyUpdate( enabled );
}

uint8_t DeviceInfo::getOptionBiokeyUpdate()
{
    return m_firmwareOptions.getOptionBiokeyUpdate();
}

void DeviceInfo::setSetupOptions( uint16_t mask, uint16_t options )
{
    m_firmwareOptions.setSetupOptions( mask, options );
}

uint16_t DeviceInfo::getSetupOptions( uint16_t mask )
{
    return m_firmwareOptions.getSetupOptions( mask );
}

void DeviceInfo::setPayableOptions( uint8_t mask, uint8_t options )
{
    m_firmwareOptions.setPayableOptions( mask, options );
}

bool DeviceInfo::isPayableOption( FirmwareOptions::EnumPayableOption option )
{
    return m_firmwareOptions.isPayableOption( option );
}

